//
// command example: 
// topic: command/plugwise
// payload: {"address":"E63D8F", "command":"on"}
//

var CIRCLEMACPREFIX = "000D6F0000";
var circleMap = {
"138BF6": {"id":"pw01", "onvalue": 0, wh:0, watt:0},          // Server LAN DSL
"236AB6": {"id":"pw02", "onvalue": 0, wh:0, watt:0},          // fridge kitchen
"236B02": {"id":"pw03", "onvalue":25, wh:0, watt:0},          // dishwasher
"236D1D": {"id":"pw04", "onvalue":10, wh:0, watt:0},          // fridge garage
"2362D7": {"id":"pw05", "onvalue":10, wh:0, watt:0},          // freezer garage
"2362B8": {"id":"pw06", "onvalue": 4, wh:0, watt:0},          // microwave
"236D7E": {"id":"pw07", "onvalue":25, wh:0, watt:0},          // floor pump
/*"236A2F": {"id":"pw08", "onvalue":0 },                      // kapot */
"236878": {"id":"pw09", "onvalue": 7, wh:0, watt:0},          // camera ledbar
"236A48": {"id":"pw10", "onvalue":25, wh:0, watt:0},          // washing machine
"23723F": {"id":"pw11", "onvalue":10, wh:0, watt:0},          // garage door opener
"2372BA": {"id":"pw12", "onvalue":25, wh:0, watt:0}           // wash dryer
}

var tools = require('./tools');
var events = require('events');
var util = require('util');
var plugwiseApi = require('plugwisejs');
var plugwise = null;
var queryQueue = [];
var commandQueue = [];

tools.initProcess('plugwise');

// connect to MQTT broker, get settings.
(function connect() {
    setInterval(function() {
      tools.log('MEMX Before:'+util.inspect(process.memoryUsage()));
      global.gc();
      tools.log('MEMX After :'+util.inspect(process.memoryUsage()));  
    }, 3600*1000);    
    tools.clog('Connecting...');
    tools.mqttClient.connect({clientId: 'drv_'+tools.systemId});
})();

tools.mqttClient.on('connect', function(packet) { 
  tools.clog('Connected to broker, getting settings');
  tools.settings.require('port');
  tools.settings.require('interval');
});

tools.settings.on('settingscomplete', function() {
  setupDriver();
});

function setupDriver() {
	tools.clog("Setting up driver"); 
  plugwise = plugwiseApi.init({log: 1, serialport: tools.settings.get('port')});  
  for (var circle in circleMap) {
    circleMap[circle].device = plugwise(CIRCLEMACPREFIX+circle);
  } 
  tools.mqttClient.subscribe("command/"+tools.systemId);
  tools.mqttClient.on('message', function(packet) {
    switch(true) {
      case packet.topic === "command/"+tools.systemId:
        cmnd = packet.payload;
        tools.log(packet.payload);
        commandQueue.push(cmnd);
        processQueues();
        break;
      default:
        tools.log("unhandled message:"+JSON.stringify(packet));
    }
  });
  
  info();
  setInterval(function() {query();}, tools.settings.get('interval')*1000);
}

function query() {  
  queryQueue.push('{"command":"query"}');
  processQueues();
}

function info() {  
  queryQueue.push('{"command":"info"}');
  processQueues();
}
    
function processQueues() {  
  if (commandQueue.length > 0) {   
    var cmd = commandQueue[0];
    commandQueue.splice(0,1);
    processCommand(cmd);
  } else {
    if (queryQueue.length > 0) {
      var cmd = queryQueue[0];
      queryQueue.splice(0,1);
      processCommand(cmd);      
    }
  }
}

function queryCircle(i) {
  var a = circleMap[i].device.powerinfo(function(result) {
    tools.log('***'+circleMap[i].id);
    tools.log(util.inspect(circleMap[i].device));
    tools.log(util.inspect(result));
    
    if ('watt' in result) {
      var wh = (tools.settings.get('interval')/3600)*result.watt;
      if (result.watt >= 0) {
        circleMap[i].watt = result.watt;
        circleMap[i].wh  += wh;
        tools.newValue(i+'/on', circleMap[i].device.data.relay?1:0);
        tools.newValue(i+'/active', result.watt > circleMap[i].onvalue?1:0);
        tools.newValue(i+'/powerusage', Math.round(circleMap[i].watt*1000)/1000);
        //tools.newValue(i+'/powertotal', circleMap[i].wh); not used
      } else {
      }
    } else {
      //tools.logger.warn('no power info found');
    }
    //console.dir(circleMap[i]);
  });
}

function processCommand(cmnd) {
  tools.log(cmnd);
  var action = JSON.parse(cmnd);
  switch(true) {
    case((action.command === 'on')||(action.command === 'off')):
      d = circleMap[action.address.toUpperCase()].device;
      if (d) {
        switch(true) {
          case action.command === 'on':
            d.poweron();
            break;
          case action.command === 'off':
            d.poweroff();
            break;
          default:
            tools.log('Command '+action.command+' not found!');
            return;
            break;          
        }
      }
      break;
    case(action.command === 'info'):
      for (var circle in circleMap) {
        (function(circle) {
          circleMap[circle].device.info(function(result) {
            tools.log('***I '+circleMap[circle].id);
            tools.log(util.inspect(circleMap[circle].device));
            tools.log(util.inspect(result));
          });
        })(circle);
      }
      break;
      
    case(action.command === 'query'):
      for (var c in circleMap) {
        queryCircle(c);
      }
      break;
    default:
      tools.log('Command '+action.command+' not found!');
      return;
      break;
  }
}  
